figma.showUI(__html__, {
  height: 300
});

// Figmaからフォントリストを取得してUIに送信
(async () => {
  const fonts = await figma.listAvailableFontsAsync();
  figma.ui.postMessage({init: fonts});
})();

// UIからフォント変更受信
figma.ui.onmessage = async (msg) => {
  if (msg.type === 'change') {
    if (msg.fontName === 'Mixed') {
      return;
    }
    const selectedTextRange = figma.currentPage.selectedTextRange;
    if (selectedTextRange) {
      if (selectedTextRange.start !== selectedTextRange.end) {
        await figma.loadFontAsync(msg.fontName);
        for (let i = 0; i < selectedTextRange.node.characters.length; i++) {
          await figma.loadFontAsync(selectedTextRange.node.getRangeFontName(i, i + 1));
        }
        selectedTextRange.node.setRangeFontName(selectedTextRange.start, selectedTextRange.end, msg.fontName);
      }
    } else {
      for (const node of figma.currentPage.selection) {
        if (node.type === 'TEXT') {
          await figma.loadFontAsync(msg.fontName);
          node.fontName = msg.fontName;
        }
      }
    }
  }
};

// フォント変更をUIに送信
figma.on('selectionchange', () => {
  const fontNameList = [];
  const selectedTextRange = figma.currentPage.selectedTextRange;
  if (selectedTextRange && selectedTextRange.start) {
    if (selectedTextRange.start !== selectedTextRange.end) {
      const fontName = selectedTextRange.node.getRangeFontName(selectedTextRange.start, selectedTextRange.end);
      if (fontName !== figma.mixed) {
        fontNameList.push(fontName);
      }
    } else {
      if (selectedTextRange.start !== 0) {
        const fontName = selectedTextRange.node.getRangeFontName(selectedTextRange.start - 1, selectedTextRange.end);
        if (fontName !== figma.mixed) {
          fontNameList.push(fontName);
        }
      } else {
        const fontName = selectedTextRange.node.getRangeFontName(selectedTextRange.start, selectedTextRange.end + 1);
        if (fontName !== figma.mixed) {
          fontNameList.push(fontName);
        }
      }
    }
  } else {
    for (const node of figma.currentPage.selection) {
      if (node.type === 'TEXT') {
        for (let i = 0; i < node.characters.length; i++) {
          fontNameList.push(node.getRangeFontName(i, i + 1));
        }
      }
    }
  }
  const familyList = fontNameList.map((item) => {
    return item.family;
  });
  const setList = fontNameList.filter((item, index) => {
    return familyList.indexOf(item.family) === index;
  });
  if (setList.length === 1) {
    figma.ui.postMessage({font: setList[0]});
  } else {
    figma.ui.postMessage({font: null});
  }
});
